/*
    libNZ - experimental cryptography library
    Copyright (C) 2021  Sylvain Saucier

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#![allow(dead_code)]

use libnzbig::Big;  
use libnzprimes::Primes;

pub struct Hash {
  pub accumulator:Big,
  counter:u64,
  pub bytes:usize,   //requested size of the hash in bytes
}
impl Hash {

  pub fn new(size:usize) -> Hash {
    let sizeof_accumulator = size / 4 + if size % 4 > 0 {1} else {0};
    let mut my = Hash { accumulator:Big::new(sizeof_accumulator), counter:0, bytes:size };
    my.accumulator.add(1);
    return my
  }

  pub fn digest(&mut self, mut value:u32){
    self.counter += 1;
    value += self.counter as u32;
    for x in 0..self.accumulator.segments.len() {
      value += x as u32;
      self.accumulator.shift(31);
      self.accumulator.add( value.rotate_left(x as u32) );
    }
  }
  
  pub fn get(&mut self) -> Vec<u8> {
    let mut answer = vec![0u8;self.bytes];
    let mut big = Big::new( self.accumulator.segments.len() );
    big.add(1);
    let mut primes = Primes::new();

    for x in 0..self.accumulator.segments.len() {
      for y in 0..32 {
        let mut offset = (64 * x) + (2 * y);
        let mask = 1u64 << y;
        if mask & self.accumulator.segments[x] > 0 {
          offset += 1;
        }
        big.multiply(primes.get(offset));
      }
    }

    for x in 0..self.bytes {
      //code using proper big endian conversion and byte convefrsion...
      answer[ x ] = ( big.segments[ x/4 ] >> (24 - ( (x % 4) * 8) )) as u8;
    }

    return answer;
  }
}
